﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using iX.MeinWeinkeller.DAO;

namespace iX.MeinWeinkeller.UI
{
    /// <summary>
    /// Weine anzeigen und editieren
    /// </summary>
    public partial class Wein_Default : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                SuchergebnisseVorbereiten(); // Suchergebnisse in das Grid View
                WeingutlisteFuellen();       // Weingüter in die Drop Down Liste
            }
        }

        /// <summary>
        /// Liste der Weingüter in die Dropdownliste
        /// </summary>
        private void WeingutlisteFuellen()
        {
            IQueryable<Weingut> wgl = WeingutDAO.ReadAll();
            ddlWeingut.DataSource = wgl;
            ddlWeingut.DataBind();
        }

        /// <summary>
        /// Suchergebnisse in das Gridview
        /// </summary>
        private void SuchergebnisseVorbereiten()
        {
            if (Session["WeinSuchstring"] != null)
            {
                tbSuche.Text = Session["WeinSuchstring"].ToString();
                Suche(tbSuche.Text);
            }
        }

        /// <summary>
        /// Bildschirmfelder mit Daten aus einem Wein füllen
        /// </summary>
        /// <param name="wein">Wein, der anzuzeigen ist</param>
        private void BildschirmFuellen(Wein wein)
        {
            // Verstecktes Feld mit id füllen
            hfWeinID.Value = wein.id.ToString();

            // Anzeigefelder füllen
            lblWeinName.Text = wein.Name;
            lblJahrgang.Text = wein.Jahrgang.ToString();
            lblPreis.Text = wein.Preis.ToString();
            lblWeingut.Text = wein.Weingut.Name + ", "  + wein.Weingut.Ort;
            lblBeschreibung.Text = wein.Beschreibung.Replace("\r\n","<br />");

            // Editierfelder füllen
            tbWeinName.Text = wein.Name;
            tbJahrgang.Text = wein.Jahrgang.ToString();
            tbPreis.Text = wein.Preis.ToString();
            TaBeschreibung.Value = wein.Beschreibung;
            ddlWeingut.SelectedValue = wein.WeingutID.ToString();

            // Bild anzeigen
            string file = FileDAO.GetFile(ImagePath(), wein.id);
            if (file != null)
            {
                imgWein.ImageUrl = "~/images/wein/" + file;
                imgWein.Visible = true;
            }
            else
            {
                imgWein.Visible = false;
            }

            // Grid View mit den Weinlagern füllen
            LagerFuellen();
        }

        /// <summary>
        /// Suche nach einem Wein wurde ausgelöst
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnSuche_Click(object sender, EventArgs e)
        {
            Suche(tbSuche.Text);  // Weine suchen
        }
        
        /// <summary>
        /// Nach Weinen suchen und Ergebnisliste an das Grid View binden
        /// </summary>
        /// <param name="strSuchstring">Suchstring</param>
        private void Suche(string strSuchstring)
        {
            // Suchen
            Session["WeinSuchstring"] = strSuchstring;
            IQueryable<Wein> weine = WeinDAO.Read(strSuchstring);

            // An das Grid View binden
            gvSuchergebnis.DataSource = weine;
            gvSuchergebnis.DataBind();           
        }
        
        /// <summary>
        /// Anderer Wein wurde aus der Suchliste gewählt
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void gvSuchergebnis_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Wein lesen
            int id = (int) gvSuchergebnis.SelectedDataKey.Value;
            Wein wein = WeinDAO.ReadSingle(id);

            // Wein anzeigen
            BildschirmFuellen(wein);
            mv1.ActiveViewIndex = VI_ANZEIGEN;
        }
        
        /// <summary>
        /// Wein soll editiert werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbBearbeiten_Click(object sender, EventArgs e)
        {
            // Editier-View anzeigen
            mv1.ActiveViewIndex = VI_BEARBEITEN;
        }
        
        /// <summary>
        /// Editierter Wein soll gespiechert werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnSpeichern_Click(object sender, EventArgs e)
        {
            int id = -1;
            Wein wein;
            if (hfWeinID.Value == "")
            {
                // Neuen Wein erzeugen
                id = WeinDAO.Create(tbWeinName.Text, TaBeschreibung.Value, Convert.ToInt32(tbJahrgang.Text), Convert.ToDecimal(tbPreis.Text), Convert.ToInt32(ddlWeingut.SelectedValue));
            }
            else
            {
                // Vorhandenen Wein aktualisieren
                id = Convert.ToInt32(hfWeinID.Value);
                WeinDAO.Update(Convert.ToInt32(hfWeinID.Value), tbWeinName.Text, TaBeschreibung.Value, Convert.ToInt32(tbJahrgang.Text), Convert.ToDecimal(tbPreis.Text), Convert.ToInt32(ddlWeingut.SelectedValue));
            }
            // ggf. Datei speichern
            if (fuWein.HasFile)
            {
                FileDAO.SaveFile(ImagePath(), id, fuWein);
            }

            // Änderungen anzeigen
            wein = WeinDAO.ReadSingle(id);
            BildschirmFuellen(wein);
            SuchergebnisseVorbereiten();

            mv1.ActiveViewIndex = VI_ANZEIGEN;
        }
        
        /// <summary>
        /// Bearbeiten soll abgebrochen werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnAbbrechen_Click(object sender, EventArgs e)
        {
            if (hfWeinID.Value == "")
            {
                // Neuer Wein
                hfWeinID.Value = "";
                lblWeinName.Text = "";
                mv1.ActiveViewIndex = VI_LISTE;
            }
            else
                mv1.ActiveViewIndex = VI_ANZEIGEN;
        }
        
        /// <summary>
        /// Es soll ein neuer Wein angelegt werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbNeu_Click(object sender, EventArgs e)
        {
            hfWeinID.Value = "";
            lblWeinName.Text = "Neuer Wein";
            mv1.ActiveViewIndex = VI_BEARBEITEN;
        }
        
        /// <summary>
        /// Wein soll gelöscht werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbLoeschen_Click(object sender, EventArgs e)
        {
            // Wein löschen
            int id = Convert.ToInt32(hfWeinID.Value);
            WeinDAO.Delete(id);

            // Geänderte Daten anzeigen
            hfWeinID.Value = "";
            lblWeinName.Text = "";
            imgWein.Visible = false;
            SuchergebnisseVorbereiten();
            mv1.ActiveViewIndex = VI_LISTE;
        }
        
        /// <summary>
        /// Bild soll gelöscht werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbBildLoeschen_Click(object sender, EventArgs e)
        {
            // Bild löschen
            int id = Convert.ToInt32(hfWeinID.Value);
            FileDAO.DeleteFile(ImagePath(), id);
            imgWein.Visible = false;
        }

        /// <summary>
        /// Grid View mit den Weinlagern dieses Weins füllen
        /// </summary>
        protected void LagerFuellen()
        {
            // Weinlager des Weins holen
            IQueryable<Weinlager> wl = WeinlagerDAO.ReadByWein(Convert.ToInt32(hfWeinID.Value));

            // ... und an das DataGrid binden
            gvLagerort.DataSource = wl;
            gvLagerort.DataBind();

            // Noch mögliche Weinkeller holen
            WeinkellerListeFuellen();
        }

        /// <summary>
        /// Neues Weinlager soll angelegt werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbNeuLager_Click(object sender, EventArgs e)
        {
            // Editier-Panel anzeigen
            pnlLagerort.Visible = true;
            ddlWeinkeller.Visible = true;
            lblWeinkeller.Text = "Weinkeller:";
            WeinkellerListeFuellen();
        }

        /// <summary>
        /// Neues oder geändertes Lager soll gespeichert werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void LagerSpeichern_Click(object sender, EventArgs e)
        {
            if (hfLager.Value == "")
            {
                // Neues Weinlager erzeugen
                WeinlagerDAO.Create(Convert.ToInt32(ddlWeinkeller.SelectedValue), Convert.ToInt32(hfWeinID.Value), Convert.ToInt32(tbFlaschenzahl.Text));
                LagerFuellen();
            }
            else
            {
                // Bestehendes Weinlager sichern
                int id = Convert.ToInt32(hfLager.Value);
                Weinlager wl = WeinlagerDAO.ReadSingle(id);
                WeinlagerDAO.Update(id, Convert.ToInt32(tbFlaschenzahl.Text));
            }

            // Editier-Panel verstecken
            hfLager.Value = "";
            pnlLagerort.Visible = false;

            // Lagerliste neu aufbauen
            LagerFuellen();
        }

        /// <summary>
        /// Bearbeiten eines Weinlagers soll abgebrochen werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void LagerAbbrechen_Click(object sender, EventArgs e)
        {
            // Editier-Panel verstecken
            hfLager.Value = "";
            pnlLagerort.Visible = false;

        }

        /// <summary>
        /// Lagerort soll gelöscht werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void gvLagerort_RowDeleting(object sender, GridViewDeleteEventArgs e)
        {
            // Den selektierten Index ermitteln
            int index = e.RowIndex;
            gvLagerort.SelectedIndex = index;

            // Weinlager löschen
            WeinlagerDAO.Delete(Convert.ToInt32(gvLagerort.SelectedDataKey.Value));
            LagerFuellen();
        }

        /// <summary>
        /// Lagerort soll bearbeitet werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void gvLagerort_RowEditing(object sender, GridViewEditEventArgs e)
        {
            // Die Bildschirmelemente sichtbar machen
            ddlWeinkeller.Visible = false;
            pnlLagerort.Visible = true;

            // Den selektierten Index ermitteln
            int index = e.NewEditIndex;
            gvLagerort.SelectedIndex = index;

            // Weinlager lesen
            Weinlager wl = WeinlagerDAO.ReadSingle(Convert.ToInt32(gvLagerort.SelectedDataKey.Value));

            // Weinlager anzeigen
            tbFlaschenzahl.Text = wl.Flaschenzahl.ToString();
            lblWeinkeller.Text = wl.Weinkeller.Name + ", " + wl.Weinkeller.Ort;

            // Den gewählten Lagerort in das versteckte Feld schreiben
            hfLager.Value = gvLagerort.SelectedDataKey.Value.ToString();
        }

        /// <summary>
        /// Drop Down Liste mit den Weinkellern füllen,
        /// in denen dieser Wein noch nicht lagert
        /// </summary>
        private void WeinkellerListeFuellen()
        {
            // Die Weinkeller ermitteln, in denen der Wein noch nicht gelagert ist
            IQueryable<Weinkeller> wks = WeinkellerDAO.ReadEligible(Convert.ToInt32(hfWeinID.Value));

            // Die Weinkeller an die Drop-Down-Liste binden
            ddlWeinkeller.DataSource = wks;
            ddlWeinkeller.DataBind();
            if (ddlWeinkeller.Items.Count == 0)
            {
                lbNeuLager.Enabled = false;
            }
            else
            {
                lbNeuLager.Enabled = true;
            }
        }

        /// <summary>
        /// Pfad zur Bilddatei
        /// </summary>
        /// <returns></returns>
        private string ImagePath()
        {
            return MapPath("~/images/wein");
        }

        private const int VI_LISTE = -1;
        private const int VI_ANZEIGEN = 0;
        private const int VI_BEARBEITEN = 1;
    }
}
