﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using iX.MeinWeinkeller.DAO;

namespace iX.MeinWeinkeller.UI
{
    public partial class Weingut_Default : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                // Liste mit Suchergebnissen füllen
                SuchergebnisseVorbereiten();
            }
        }

        /// <summary>
        /// Grid View mit Suchergebnissen füllen
        /// </summary>
        private void SuchergebnisseVorbereiten()
        {
            // Alten Suchstring holen
            if (Session["WeingutSuchstring"] != null)
            {
                tbSuche.Text = Session["WeingutSuchstring"].ToString();
                Suche(tbSuche.Text);
            }
        }
        
        /// <summary>
        /// Suchen und Suchergebnisse an Grid View binden
        /// </summary>
        /// <param name="strSuchstring"></param>
        private void Suche(string strSuchstring)
        {
            Session["WeingutSuchstring"] = strSuchstring;
            IQueryable<Weingut> wgs = WeingutDAO.Read(strSuchstring);

            gvSuchergebnis.DataSource = wgs;
            gvSuchergebnis.DataBind();
        }
        
        /// <summary>
        /// Daten des Weinguts anzagein
        /// </summary>
        /// <param name="wg">das Weingut, das angezeigt werden soll</param>
        private void BildschirmFuellen(Weingut wg)
        {
            // Verstecktes Feld mit id füllen
            hfWeingutID.Value = wg.id.ToString();

            // Anzeigefelder füllen
            lblWeingutName.Text = wg.Name;
            lblAdresse.Text = wg.Anrede + " " + wg.Vorname + " " + wg.Nachname + ", " + wg.Strasse + ", " + wg.Land + " " + wg.Postleitzahl + " " + wg.Ort;
            lblBeschreibung.Text = wg.Beschreibung.Replace("\r\n","<br />");

            // Editierfelder füllen
            tbWeingutName.Text = wg.Name;
            ddlAnrede.SelectedValue = wg.Anrede;
            tbVorname.Text = wg.Vorname;
            tbNachname.Text = wg.Nachname;
            tbStrasse.Text = wg.Strasse;
            tbPostleitzahl.Text = wg.Postleitzahl;
            tbOrt.Text = wg.Ort;
            ddlLand.SelectedValue = wg.Land;
            TaBeschreibung.Value = wg.Beschreibung;

            // Bild anzeigen
            string file = FileDAO.GetFile(ImagePath(), wg.id);
            if (file != null)
            {
                imgWeingut.ImageUrl = "~/images/weingut/" + file;
                imgWeingut.Visible = true;
            }
            else
            {
                imgWeingut.Visible = false;
            }
        }
        
        /// <summary>
        /// Nach einem Weingut suchen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnSuche_Click(object sender, EventArgs e)
        {
            Suche(tbSuche.Text);
        }
        

        /// <summary>
        /// Es wurde ein Weingut aus der Ergebnisliste ausgewählt
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void gvSuchergebnis_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Weingut lesen
            int id = (int) gvSuchergebnis.SelectedDataKey.Value;
            Weingut wg = WeingutDAO.ReadSingle(id);

            // Weingut anzeigen
            BildschirmFuellen(wg);
            mv1.ActiveViewIndex = VI_ANZEIGEN;
        }
        
        /// <summary>
        /// Das Weingut soll bearbeitet werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbBearbeiten_Click(object sender, EventArgs e)
        {
            // Editier-View anzeigen
            mv1.ActiveViewIndex = VI_BEARBEITEN;
        }
        
        /// <summary>
        /// Neues oder geändertes Weingut soll gespeichert werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnSpeichern_Click(object sender, EventArgs e)
        {
            int id = -1;
            Weingut wg;

            if (hfWeingutID.Value == "")
            {
                // Neuen Datensatz erzeugen
                id = WeingutDAO.Create(tbWeingutName.Text, TaBeschreibung.Value, ddlAnrede.SelectedValue, tbVorname.Text, tbNachname.Text, tbStrasse.Text, tbPostleitzahl.Text, tbOrt.Text, ddlLand.SelectedValue);
            }
            else
            {
                // Bestehenden Datensatz speichern
                id = Convert.ToInt32(hfWeingutID.Value);
                WeingutDAO.Update(Convert.ToInt32(hfWeingutID.Value), tbWeingutName.Text, TaBeschreibung.Value, ddlAnrede.SelectedValue, tbVorname.Text, tbNachname.Text, tbStrasse.Text, tbPostleitzahl.Text, tbOrt.Text, ddlLand.SelectedValue);
            }
            if (fuWeingut.HasFile)
            {
                // Bilddatei speichern
                FileDAO.SaveFile(ImagePath(), id, fuWeingut);
            }

            // Geänderte Daten anzeigen
            wg = WeingutDAO.ReadSingle(id);
            BildschirmFuellen(wg);
            SuchergebnisseVorbereiten();

            mv1.ActiveViewIndex = VI_ANZEIGEN;
        }
        
        /// <summary>
        /// Bearbeiten soll abgebrochen werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnAbbrechen_Click(object sender, EventArgs e)
        {
            if (hfWeingutID.Value == "")
            {
                hfWeingutID.Value = "";
                lblWeingutName.Text = "";
                mv1.ActiveViewIndex = VI_LISTE;
            }
            else
                mv1.ActiveViewIndex = VI_ANZEIGEN;
        }
        
        /// <summary>
        /// Ein neues Weingut soll angelegt werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbNeu_Click(object sender, EventArgs e)
        {
            hfWeingutID.Value = "";
            lblWeingutName.Text = "Neues Weingut";
            mv1.ActiveViewIndex = VI_BEARBEITEN;
        }
        
        /// <summary>
        /// Das Weingut soll gelöscht werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbLoeschen_Click(object sender, EventArgs e)
        {
            // Weingut löschen
            int id = Convert.ToInt32(hfWeingutID.Value);
            WeingutDAO.Delete(id);

            // Geänderte Daten anzeigen
            hfWeingutID.Value = "";
            lblWeingutName.Text = "";
            imgWeingut.Visible = false;
            SuchergebnisseVorbereiten();
            mv1.ActiveViewIndex = VI_LISTE;
        }
        
        /// <summary>
        /// Das Bild des Weinguts soll gelöscht werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbBildLoeschen_Click(object sender, EventArgs e)
        {
            // Bild löschen
            int id = Convert.ToInt32(hfWeingutID.Value);
            FileDAO.DeleteFile(ImagePath(), id);
            imgWeingut.Visible = false;
        }

        /// <summary>
        /// Pfad zum Speicherort der Bilddateien für Weingüter
        /// </summary>
        /// <returns></returns>
        private string ImagePath()
        {
            return MapPath("~/images/weingut");
        }

        private const int VI_LISTE = -1;
        private const int VI_ANZEIGEN = 0;
        private const int VI_BEARBEITEN = 1;
    }
}
