﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using iX.MeinWeinkeller.DAO;

namespace iX.MeinWeinkeller.UI
{
    public partial class Weinkeller_Default : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                WeinkellerFuellen();  // Grid View mit allen Weinkellern füllen
            }
        }

        /// <summary>
        /// Grid View mit allen Weinkellern füllen
        /// </summary>
        private void WeinkellerFuellen()
        {
            IQueryable<Weinkeller> weinkeller = WeinkellerDAO.ReadAll();

            gvSuchergebnis.DataSource = weinkeller;
            gvSuchergebnis.DataBind();
        }

        /// <summary>
        /// Einen Weinkeller anzeigen
        /// </summary>
        /// <param name="weinkeller">Der Weinkeller, der angezeigt werden soll</param>
        private void BildschirmFuellen(Weinkeller weinkeller)
        {
            // Verstecktes Feld mit id füllen
            hfWeinkellerID.Value = weinkeller.id.ToString();

            // Anzeigefelder füllen
            lblWeinkellerName.Text = weinkeller.Name;
            lblBeschreibung.Text = weinkeller.Name + ", " + weinkeller.Ort;

            // Editierfelder füllen
            tbWeinkellerName.Text = weinkeller.Name;
            tbOrt.Text = weinkeller.Ort;

            // Bild anzeigen
            string file = FileDAO.GetFile(ImagePath(), weinkeller.id);
            if (file != null)
            {
                imgWeinkeller.ImageUrl = "~/images/weinkeller/" + file;
                imgWeinkeller.Visible = true;
            }
            else
            {
                imgWeinkeller.Visible = false;
            }
        }

        /// <summary>
        /// Es wurde ein Weinkeller ausgewählt
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void gvSuchergebnis_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Weinkeller lesen
            int id = (int) gvSuchergebnis.SelectedDataKey.Value;
            Weinkeller weinkeller = WeinkellerDAO.ReadSingle(id);

            // und anzeigen
            BildschirmFuellen(weinkeller);
            mv1.ActiveViewIndex = VI_ANZEIGEN;
        }

        /// <summary>
        /// Der Weinkeller soll bearbeitet werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbBearbeiten_Click(object sender, EventArgs e)
        {
            mv1.ActiveViewIndex = VI_BEARBEITEN;
        }

        /// <summary>
        /// Der Weinkeller soll gespeichert werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnSpeichern_Click(object sender, EventArgs e)
        {
            int id = -1;
            Weinkeller weinkeller;
            if (hfWeinkellerID.Value == "")
            {
                // Neuen Weinkeller anlegen
                id = WeinkellerDAO.Create(tbWeinkellerName.Text, tbOrt.Text);
            }
            else
            {
                // Bestehenden Weinkeller aktualisieren
                id = Convert.ToInt32(hfWeinkellerID.Value);
                WeinkellerDAO.Update(Convert.ToInt32(hfWeinkellerID.Value), tbWeinkellerName.Text, tbOrt.Text);
            }
            // Bild hochladen
            if (fuWeinkeller.HasFile)
            {
                FileDAO.SaveFile(ImagePath(), id, fuWeinkeller);
            }
            // Neue Daten anzeigen
            weinkeller = WeinkellerDAO.ReadSingle(id);
            BildschirmFuellen(weinkeller);
            WeinkellerFuellen();

            mv1.ActiveViewIndex = VI_ANZEIGEN;
        }

        /// <summary>
        /// Bearbeiten soll abgebrochen werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnAbbrechen_Click(object sender, EventArgs e)
        {
            if (hfWeinkellerID.Value == "")
            {
                hfWeinkellerID.Value = "";
                lblWeinkellerName.Text = "";
                mv1.ActiveViewIndex = VI_LISTE;
            }
            else
                mv1.ActiveViewIndex = VI_ANZEIGEN;
        }

        /// <summary>
        /// Ein neuer Weinkeller soll angelegt werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbNeu_Click(object sender, EventArgs e)
        {
            hfWeinkellerID.Value = "";
            lblWeinkellerName.Text = "Neuer Weinkeller";
            mv1.ActiveViewIndex = VI_BEARBEITEN;
        }

        /// <summary>
        /// Der Weinkeller soll gelöscht werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbLoeschen_Click(object sender, EventArgs e)
        {
            // Weinkeller löschen
            int id = Convert.ToInt32(hfWeinkellerID.Value);
            WeinkellerDAO.Delete(id);

            // Geänderte Daten anzeigen
            hfWeinkellerID.Value = "";
            lblWeinkellerName.Text = "";
            imgWeinkeller.Visible = false;
            WeinkellerFuellen();
            mv1.ActiveViewIndex = VI_LISTE;
        }

        /// <summary>
        /// Das Bild des Weinkellers soll gelöscht werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbBildLoeschen_Click(object sender, EventArgs e)
        {
            // Bild löschen
            int id = Convert.ToInt32(hfWeinkellerID.Value);
            FileDAO.DeleteFile(ImagePath(), id);
            imgWeinkeller.Visible = false;
        }

        /// <summary>
        /// Pfad zum Speicherort der Bilddateien von Weinkellern zurückgeben
        /// </summary>
        /// <returns></returns>
        private string ImagePath()
        {
            return MapPath("~/images/weinkeller");
        }

        private const int VI_LISTE = -1;
        private const int VI_ANZEIGEN = 0;
        private const int VI_BEARBEITEN = 1;
}
}
